import argparse
import logging
import os.path

import sqs_event.setup as setup
from sqs_event.dispatcher import dispatch, HANDLERS
from sqs_event.normalize import normalize
import sqs_event.queue as queue
import sqs_event.user as user


logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)


def init(config):
    queue.setup(config)
    user.setup(config)


def run():
    while True:
        try:
            message = queue.recv()
            try:
                event = normalize(message.body)
                dispatch(event)
            except Exception:
                log.exception('unable to handle message={}'.format(message.id))
            else:
                queue.delete(message)
        except KeyboardInterrupt:
            log.info('Received shutdown command from keyboard')
            break
        except Exception:
            log.exception('exception encountered')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        '--config', default='config',
        help='name of the config file',
    )
    parser.add_argument(
        '-c', '--config_only', default=False, action='store_true',
        help='display config values and stop',
    )
    parser.add_argument(
        '--event', default='event',
        help='name of the event definition file',
    )
    parser.add_argument(
        '--events_only', default=False, action='store_true',
        help='display event handlers and stop',
    )
    args = parser.parse_args()

    setup.load(args.event)
    if os.path.exists(args.config):
        try:
            setup.CONFIG._load(args.config)
        except Exception as e:
            raise Exception(e)
    if args.config_only:
        print(setup.CONFIG)
    elif args.events_only:
        print(HANDLERS)
    else:
        init(setup.CONFIG)
        log.info('listening on queue: {}'.format(queue.QUEUE.url))
        run()



import boto3
import json
import logging

from ergaleia.config import Mini


log = logging.getLogger(__name__)


QUEUE = Mini(
    'url',
    'wait_time validator=int',
    'visibility_timeout validator=int',
)


SQS = boto3.client('sqs')


def setup(config):
    url = SQS.get_queue_url(QueueName=config.queue.name)['QueueUrl']
    QUEUE.set(
        url=url,
        wait_time=config.queue.wait_time,
        visibility_timeout=config.queue.visibility_timeout,
    )


def recv():
    if not QUEUE.url:
        raise Exception('queue has not been setup')
    while True:
        sqs_message = SQS.receive_message(
            QueueUrl=QUEUE.url,
            WaitTimeSeconds=QUEUE.wait_time,
            VisibilityTimeout=QUEUE.visibility_timeout,
        )
        message = Message.parse(sqs_message)
        if message is not None:
            return message


def delete(message):
    SQS.delete_message(
        QueueUrl=QUEUE.url,
        ReceiptHandle=message.handle,
    )


class Message(object):

    def __init__(self, body, id, handle):
        self.body = body
        self.id = id
        self.handle = handle

    def __repr__(self):
        return json.dumps(self.body, indent=4)

    @classmethod
    def parse(cls, data):
        if 'Messages' not in data:
            return None
        try:
            data = data['Messages'][0]
            body = json.loads(data['Body'])
            handle = data['ReceiptHandle']
            id = data['MessageId']
        except Exception:
            log.exception('unable to parse message')
            return None
        else:
            return Message(body, id, handle)



from sqs_event.event import Event


def normalize(data):

    # command/kwargs style
    try:
        command = data['command']
        return Event(command, **data['data'])
    except Exception:
        pass

    # name/args/kwargs style
    try:
        name = data['name']
        return Event(name, *data['args'], **data['kwargs'])
    except Exception:
        pass

    # AWS S3
    try:
        aws = data['Records'][0]
        name = aws['eventSource']
        if name != 'aws:s3':
            raise Exception('unhandled AWS event type: {}'.format(name))
        s3 = aws['s3']
        return Event(
            'S3',
            bucket=s3['bucket']['name'],
            key=s3['object']['key'],
        )
    except Exception:
        pass

    raise Exception('unable to normalize message data')



from distutils.core import setup
setup(
    name='sqs_event',
    version='0.0.1',
    packages=[
        'sqs_event',
    ],
    install_requires=['ergaleia'],
    description='Simple AWS SQS-based event queue handler',
    long_description="""
Documentation
-------------
    You can see the project and documentation at the `GitHub repo <https://github.com/robertchase/sqs_event>`_
    """,
    author='Bob Chase',
    url='https://github.com/robertchase/sqs_event',
    license='MIT',
)
